@@ -13,7 +13,7 @@
 ##############################################################################
 """Database connection support
 
-$Id: Connection.py 38051 2005-08-23 21:39:15Z tim_one $"""
+$Id: Connection.py 40785 2005-12-14 21:49:51Z tim_one $"""
 
 import logging
 import sys
@@ -40,7 +40,6 @@ from ZODB import POSException
 from ZODB.POSException import InvalidObjectReference, ConnectionStateError
 from ZODB.POSException import ConflictError, ReadConflictError
 from ZODB.serialize import ObjectWriter, ObjectReader, myhasattr
-from ZODB.utils import DEPRECATED_ARGUMENT, deprecated36
 from ZODB.utils import p64, u64, z64, oid_repr, positive_id
 
 global_reset_counter = 0
@@ -66,27 +65,22 @@ class Connection(ExportImport, object):
                IPersistentDataManager,
                ISynchronizer)
 
-    _storage = _normal_storage = _savepoint_storage = None
 
     _code_timestamp = 0
 
     ##########################################################################
     # Connection methods, ZODB.IConnection
 
-    def __init__(self, version='', cache_size=400,
-                 cache_deactivate_after=None, mvcc=True,
-                 txn_mgr=DEPRECATED_ARGUMENT,
-                 transaction_manager=None,
-                 synch=True):
+    def __init__(self, db, version='', cache_size=400):
         """Create a new Connection."""
 
-        if txn_mgr is not DEPRECATED_ARGUMENT:
-            deprecated36("use transaction_manager= instead of txn_mgr=")
-            if transaction_manager is None:
-                transaction_manager = txn_mgr
-            else:
-                raise ValueError("cannot specify both transaction_manager= "
-                                 "and txn_mgr=")
+        self._db = db
+        self._normal_storage = self._storage = db._storage
+        self.new_oid = db._storage.new_oid
+        self._savepoint_storage = None
+
+        self.transaction_manager = self._synch = self._mvcc = None
+
 
         self._log = logging.getLogger("ZODB.Connection")
         self._debug_info = ()
@@ -101,12 +95,14 @@ class Connection(ExportImport, object):
             # Unclear:  Why do we want version caches to behave this way?
 
             self._cache.cache_drain_resistance = 100
+
         self._committed = []
         self._added = {}
         self._added_during_commit = None
         self._reset_counter = global_reset_counter
         self._load_count = 0   # Number of objects unghosted
         self._store_count = 0  # Number of objects stored
+        self._creating = {}
 
         # List of oids of modified objects (to be invalidated on an abort).
         self._modified = []
@@ -118,15 +114,6 @@ class Connection(ExportImport, object):
         # Do we need to join a txn manager?
         self._needs_to_join = True
 
-        # If a transaction manager is passed to the constructor, use
-        # it instead of the global transaction manager.  The instance
-        # variable will hold a TM instance.
-        self.transaction_manager = transaction_manager or transaction.manager
-
-        # _synch is a boolean; if True, the Connection will register
-        # with the TM to receive afterCompletion() calls.
-        self._synch = synch
-
         # _invalidated queues invalidate messages delivered from the DB
         # _inv_lock prevents one thread from modifying the set while
         # another is processing invalidations.  All the invalidations
@@ -138,6 +125,9 @@ class Connection(ExportImport, object):
         # will execute atomically by virtue of the GIL.  But some storage
         # might generate oids where hash or compare invokes Python code.  In
         # that case, the GIL can't save us.
+        # Note:  since that was written, it was officially declared that the
+        # type of an oid is str.  TODO:  remove the related now-unnecessary
+        # critical sections (if any -- this needs careful thought).
 
         self._inv_lock = threading.Lock()
         self._invalidated = d = {}
@@ -157,7 +147,6 @@ class Connection(ExportImport, object):
         # If it is None, then the current revisions are acceptable.
         # If the connection is in a version, mvcc will be disabled, because
         # loadBefore() only returns non-version data.
-        self._mvcc = mvcc and not version
         self._txn_time = None
 
         # To support importFile(), implemented in the ExportImport base
@@ -166,11 +155,16 @@ class Connection(ExportImport, object):
         # to pass to _importDuringCommit().
         self._import = None
 
-        self.connections = None
+
+        self._reader = ObjectReader(self, self._cache, self._db.classFactory)
+
+        # Multi-database support
+        self.connections = {self._db.database_name: self}
+
 
     def add(self, obj):
         """Add a new object 'obj' to the database and assign it an oid."""
-        if self._storage is None:
+        if self._opened is None:
             raise ConnectionStateError("The database connection is closed")
 
         marker = object()
@@ -194,7 +188,7 @@ class Connection(ExportImport, object):
 
     def get(self, oid):
         """Return the persistent object with oid 'oid'."""
-        if self._storage is None:
+        if self._opened is None:
             raise ConnectionStateError("The database connection is closed")
 
         obj = self._cache.get(oid, None)
@@ -215,10 +209,8 @@ class Connection(ExportImport, object):
         self._cache[oid] = obj
         return obj
 
-    def cacheMinimize(self, dt=DEPRECATED_ARGUMENT):
+    def cacheMinimize(self):
         """Deactivate all unmodified objects in the cache."""
-        if dt is not DEPRECATED_ARGUMENT:
-            deprecated36("cacheMinimize() dt= is ignored.")
         self._cache.minimize()
 
     # TODO: we should test what happens when cacheGC is called mid-transaction.
@@ -233,7 +225,7 @@ class Connection(ExportImport, object):
             self.__onCloseCallbacks = []
         self.__onCloseCallbacks.append(f)
 
-    def close(self):
+    def close(self, primary=True):
         """Close the Connection."""
         if not self._needs_to_join:
             # We're currently joined to a transaction.
@@ -253,18 +245,28 @@ class Connection(ExportImport, object):
                     self._log.error("Close callback failed for %s", f,
                                     exc_info=sys.exc_info())
             self.__onCloseCallbacks = None
-        self._storage = self._savepoint_storage = self._normal_storage = None
-        self.new_oid = None
+
         self._debug_info = ()
-        self._opened = None
-        # Return the connection to the pool.
-        if self._db is not None:
-            if self._synch:
-                self.transaction_manager.unregisterSynch(self)
-            self._db._closeConnection(self)
-            # _closeConnection() set self._db to None.  However, we can't
-            # assert that here, because self may have been reused (by
-            # another thread) by the time we get back here.
+
+        if self._synch:
+            self.transaction_manager.unregisterSynch(self)
+            self._synch = None
+
+        if primary:
+            for connection in self.connections.values():
+                if connection is not self:
+                    connection.close(False)
+
+            # Return the connection to the pool.
+            if self._opened is not None:
+                self._db._returnToPool(self)
+
+                # _returnToPool() set self._opened to None.
+                # However, we can't assert that here, because self may
+                # have been reused (by another thread) by the time we
+                # get back here.
+        else:
+            self._opened = None
 
     def db(self):
         """Returns a handle to the database this connection belongs to."""
@@ -272,7 +274,7 @@ class Connection(ExportImport, object):
 
     def isReadOnly(self):
         """Returns True if the storage for this connection is read only."""
-        if self._storage is None:
+        if self._opened is None:
             raise ConnectionStateError("The database connection is closed")
         return self._storage.isReadOnly()
 
@@ -300,12 +302,31 @@ class Connection(ExportImport, object):
         """Return a Connection for the named database."""
         connection = self.connections.get(database_name)
         if connection is None:
-            new_con = self._db.databases[database_name].open()
+            new_con = self._db.databases[database_name].open(
+                transaction_manager=self.transaction_manager,
+                mvcc=self._mvcc, version=self._version, synch=self._synch,
+                )
             self.connections.update(new_con.connections)
             new_con.connections = self.connections
             connection = new_con
         return connection
 
+    def _implicitlyAdding(self, oid):
+        """Are we implicitly adding an object within the current transaction
+
+        This is used in a check to avoid implicitly adding an object
+        to a database in a multi-database situation.
+        See serialize.ObjectWriter.persistent_id.
+
+        """
+        return (self._creating.get(oid, 0)
+                or
+                ((self._savepoint_storage is not None)
+                 and
+                 self._savepoint_storage.creating.get(oid, 0)
+                 )
+                )
+
     def sync(self):
         """Manually update the view on the database."""
         self.transaction_manager.abort()
@@ -392,6 +413,7 @@ class Connection(ExportImport, object):
             self._flush_invalidations()
         self._needs_to_join = True
         self._registered_objects = []
+        self._creating.clear()
 
     # Process pending invalidations.
     def _flush_invalidations(self):
@@ -442,7 +464,7 @@ class Connection(ExportImport, object):
 
         # _creating is a list of oids of new objects, which is used to
         # remove them from the cache if a transaction aborts.
-        self._creating = []
+        self._creating.clear()
         self._normal_storage.tpc_begin(transaction)
 
     def commit(self, transaction):
@@ -465,8 +487,9 @@ class Connection(ExportImport, object):
         """Commit changes to an object"""
 
         if self._import:
-            # TODO:  This code seems important for Zope, but needs docs
-            # to explain why.
+            # We are importing an export file. We alsways do this
+            # while making a savepoint so we can copy export data
+            # directly to out storage, typically a TmpStore.
             self._importDuringCommit(transaction, *self._import)
             self._import = None
 
@@ -513,10 +536,15 @@ class Connection(ExportImport, object):
 
             if serial == z64:
                 # obj is a new object
-                self._creating.append(oid)
-                # Because obj was added, it is now in _creating, so it can
-                # be removed from _added.
-                self._added.pop(oid, None)
+
+                # Because obj was added, it is now in _creating, so it
+                # can be removed from _added.  If oid wasn't in
+                # adding, then we are adding it implicitly.
+
+                implicitly_adding = self._added.pop(oid, None) is None
+
+                self._creating[oid] = implicitly_adding
+
             else:
                 if (oid in self._invalidated
                     and not hasattr(obj, '_p_resolveConflict')):
@@ -591,8 +619,8 @@ class Connection(ExportImport, object):
 
         self._storage.tpc_abort(transaction)
 
-        # Note: If we invalidate a non-justifiable object (i.e. a
-        # persistent class), the object will immediately reread it's
+        # Note: If we invalidate a non-ghostifiable object (i.e. a
+        # persistent class), the object will immediately reread its
         # state.  That means that the following call could result in a
         # call to self.setstate, which, of course, must succeed.  In
         # general, it would be better if the read could be delayed
@@ -619,7 +647,7 @@ class Connection(ExportImport, object):
         """Disown any objects newly saved in an uncommitted transaction."""
         if creating is None:
             creating = self._creating
-            self._creating = []
+            self._creating = {}
 
         for oid in creating:
             o = self._cache.get(oid)
@@ -694,7 +722,7 @@ class Connection(ExportImport, object):
         database."""
         oid = obj._p_oid
 
-        if self._storage is None:
+        if self._opened is None:
             msg = ("Shouldn't load state for %s "
                    "when the connection is closed" % oid_repr(oid))
             self._log.error(msg)
@@ -731,10 +759,8 @@ class Connection(ExportImport, object):
         # dict update could go on in another thread, but we don't care
         # because we have to check again after the load anyway.
 
-        if (obj._p_oid in self._invalidated
-            and not myhasattr(obj, "_p_independent")
-            and not self._invalidated
-            ):
+        if (obj._p_oid in self._invalidated and
+                not myhasattr(obj, "_p_independent")):
             # If the object has _p_independent(), we will handle it below.
             self._load_before_or_conflict(obj)
             return
@@ -820,16 +846,11 @@ class Connection(ExportImport, object):
         """
         assert obj._p_jar is self
         if obj._p_oid is None:
-            # There is some old Zope code that assigns _p_jar
-            # directly.  That is no longer allowed, but we need to
-            # provide support for old code that still does it.
-
             # The actual complaint here is that an object without
             # an oid is being registered.  I can't think of any way to
             # achieve that without assignment to _p_jar.  If there is
-            # a way, this will be a very confusing warning.
-            deprecated36("Assigning to _p_jar is deprecated, and will be "
-                         "changed to raise an exception.")
+            # a way, this will be a very confusing exception.
+            raise ValueError("assigning to _p_jar is not supported")
         elif obj._p_oid in self._added:
             # It was registered before it was added to _added.
             return
@@ -867,8 +888,8 @@ class Connection(ExportImport, object):
         # return a list of [ghosts....not recently used.....recently used]
         return everything.items() + items
 
-    def _setDB(self, odb, mvcc=None, txn_mgr=DEPRECATED_ARGUMENT,
-               transaction_manager=None, synch=None):
+    def open(self, transaction_manager=None, mvcc=True, synch=True,
+             delegate=True):
         """Register odb, the DB that this Connection uses.
 
         This method is called by the DB every time a Connection
@@ -887,39 +908,37 @@ class Connection(ExportImport, object):
         register for afterCompletion() calls.
         """
 
-        if txn_mgr is not DEPRECATED_ARGUMENT:
-            deprecated36("use transaction_manager= instead of txn_mgr=")
-            if transaction_manager is None:
-                transaction_manager = txn_mgr
-            else:
-                raise ValueError("cannot specify both transaction_manager= "
-                                 "and txn_mgr=")
-
         # TODO:  Why do we go to all the trouble of setting _db and
         # other attributes on open and clearing them on close?
         # A Connection is only ever associated with a single DB
         # and Storage.
 
-        self._db = odb
-        self._normal_storage = self._storage = odb._storage
-        self.new_oid = odb._storage.new_oid
         self._opened = time()
-        if synch is not None:
-            self._synch = synch
-        if mvcc is not None:
-            self._mvcc = mvcc
-        self.transaction_manager = transaction_manager or transaction.manager
+        self._synch = synch
+        self._mvcc = mvcc and not self._version
+
+        if transaction_manager is None:
+            transaction_manager = transaction.manager
+
+        self.transaction_manager = transaction_manager
+
         if self._reset_counter != global_reset_counter:
             # New code is in place.  Start a new cache.
             self._resetCache()
         else:
             self._flush_invalidations()
-        if self._synch:
-            self.transaction_manager.registerSynch(self)
-        self._reader = ObjectReader(self, self._cache, self._db.classFactory)
 
-        # Multi-database support
-        self.connections = {self._db.database_name: self}
+        if synch:
+            transaction_manager.registerSynch(self)
+
+        if self._cache is not None:
+            self._cache.incrgc() # This is a good time to do some GC
+
+        if delegate:
+            # delegate open to secondary connections
+            for connection in self.connections.values():
+                if connection is not self:
+                    connection.open(transaction_manager, mvcc, synch, False)
 
     def _resetCache(self):
         """Creates a new cache, discarding the old one.
@@ -975,48 +994,7 @@ class Connection(ExportImport, object):
     ##########################################################################
     # DEPRECATED methods
 
-    def cacheFullSweep(self, dt=None):
-        deprecated36("cacheFullSweep is deprecated. "
-                     "Use cacheMinimize instead.")
-        if dt is None:
-            self._cache.full_sweep()
-        else:
-            self._cache.full_sweep(dt)
-
-    def getTransaction(self):
-        """Get the current transaction for this connection.
-
-        :deprecated:
-
-        The transaction manager's get method works the same as this
-        method.  You can pass a transaction manager (TM) to DB.open()
-        to control which TM the Connection uses.
-        """
-        deprecated36("getTransaction() is deprecated. "
-                     "Use the transaction_manager argument "
-                     "to DB.open() instead, or access "
-                     ".transaction_manager directly on the Connection.")
-        return self.transaction_manager.get()
-
-    def setLocalTransaction(self):
-        """Use a transaction bound to the connection rather than the thread.
-
-        :deprecated:
-
-        Returns the transaction manager used by the connection.  You
-        can pass a transaction manager (TM) to DB.open() to control
-        which TM the Connection uses.
-        """
-        deprecated36("setLocalTransaction() is deprecated. "
-                     "Use the transaction_manager argument "
-                     "to DB.open() instead.")
-        if self.transaction_manager is transaction.manager:
-            if self._synch:
-                self.transaction_manager.unregisterSynch(self)
-            self.transaction_manager = transaction.TransactionManager()
-            if self._synch:
-                self.transaction_manager.registerSynch(self)
-        return self.transaction_manager
+    # None at present.
 
     # DEPRECATED methods
     ##########################################################################
@@ -1030,10 +1008,10 @@ class Connection(ExportImport, object):
                                                self._normal_storage)
             self._storage = self._savepoint_storage
 
-        self._creating = []
+        self._creating.clear()
         self._commit(None)
-        self._storage.creating.extend(self._creating)
-        del self._creating[:]
+        self._storage.creating.update(self._creating)
+        self._creating.clear()
         self._registered_objects = []
 
         state = self._storage.position, self._storage.index.copy()
@@ -1064,7 +1042,7 @@ class Connection(ExportImport, object):
 
         # Copy invalidating and creating info from temporary storage:
         self._modified.extend(oids)
-        self._creating.extend(src.creating)
+        self._creating.update(src.creating)
 
         for oid in oids:
             data, serial = src.load(oid, src)
@@ -1132,7 +1110,7 @@ class TmpStore:
         self.position = 0L
         # index: map oid to pos of last committed version
         self.index = {}
-        self.creating = []
+        self.creating = {}
 
     def __len__(self):
         return len(self.index)
