@@ -13,14 +13,22 @@
 ##############################################################################
 """Container constraint tests
 
-$Id: test_constraints.py 25177 2004-06-02 13:17:31Z jim $
+$Id: test_constraints.py 40495 2005-12-02 17:51:22Z efge $
 """
 import unittest
-from zope.testing.doctestunit import DocTestSuite
+from zope.testing import doctest, module
+
+def setUp(test):
+    module.setUp(test, 'zope.app.container.constraints_txt')
+
+def tearDown(test):
+    module.tearDown(test, 'zope.app.container.constraints_txt')
 
 def test_suite():
     return unittest.TestSuite((
-        DocTestSuite('zope.app.container.constraints'),
+        doctest.DocTestSuite('zope.app.container.constraints'),
+        doctest.DocFileSuite('../constraints.txt',
+                             setUp=setUp, tearDown=tearDown),
         ))
 
 if __name__ == '__main__': unittest.main()
